@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
-from collections import deque
-from os import path as os_PATH
+from os import path as os_PATH, sep
 from pathlib import Path
+from queue import Queue
 from random import randint
 from re import search, sub
 from threading import Thread
@@ -30,14 +30,11 @@ class DownloadKit(object):
         """
         self._size = size
         self._missions = {}
-        self._waiting_list: deque = deque()
         self._threads = {i: None for i in range(self._size)}
-        self._mission_thread = None
-        self._manage_thread = None
-        self._info_thread = None
+        self._waiting_list: Queue = Queue()
         self._missions_num = 0
 
-        self.goal_path = str(goal_path)
+        self.goal_path = str(goal_path) if isinstance(goal_path, Path) else goal_path
         self.retry: int = 3
         self.interval: int = 5
         self.timeout: float = timeout if timeout is not None else 20
@@ -45,13 +42,49 @@ class DownloadKit(object):
 
         self.session = _get_session(session)
 
+    def __call__(self,
+                 file_url: str,
+                 goal_path: str = None,
+                 session: Session = None,
+                 rename: str = None,
+                 file_exists: str = None,
+                 post_data: Union[str, dict] = None,
+                 show_msg: bool = True,
+                 retry: int = None,
+                 interval: float = None,
+                 **kwargs) -> tuple:
+        """以阻塞的方式下载一个文件并返回结果                                                              \n
+        :param file_url: 文件网址
+        :param goal_path: 保存路径
+        :param session: 用于下载的Session对象，默认使用实例属性的
+        :param rename: 重命名的文件名
+        :param file_exists: 遇到同名文件时的处理方式，可选 'skip', 'overwrite', 'rename'，默认跟随实例属性
+        :param post_data: post方式使用的数据
+        :param show_msg: 是否打印进度
+        :param retry: 重试次数，默认跟随实例属性
+        :param interval: 重试间隔，默认跟随实例属性
+        :param kwargs: 连接参数
+        :return: 任务结果和信息组成的tuple
+        """
+        mission = self.add(file_url=file_url,
+                           goal_path=goal_path,
+                           session=session,
+                           rename=rename,
+                           file_exists=file_exists,
+                           post_data=post_data,
+                           retry=retry,
+                           interval=interval,
+                           **kwargs)
+        return self.wait(mission, show=show_msg)
+
     @property
     def size(self) -> int:
         """可同时运行的线程数"""
         return self._size
 
     @property
-    def waiting_list(self) -> deque:
+    def waiting_list(self) -> Queue:
+        """返回等待队列"""
         return self._waiting_list
 
     @property
@@ -65,9 +98,9 @@ class DownloadKit(object):
             raise ValueError("file_exists参数只能传入'skip', 'overwrite', 'rename'")
         self._file_exists = mode
 
-    def is_running(self) -> list:
+    def is_running(self) -> bool:
         """检查是否有线程还在运行中"""
-        return [k for k, v in self._threads.items() if v is not None]
+        return any(self._threads.values())
 
     def add(self,
             file_url: str,
@@ -79,7 +112,7 @@ class DownloadKit(object):
             retry: int = None,
             interval: float = None,
             **kwargs) -> 'Mission':
-        """添加一个下载任务，返回其id值
+        """添加一个下载任务并将其返回                                                                    \n
         :param file_url: 文件网址
         :param goal_path: 保存路径
         :param session: 用于下载的Session对象，默认使用实例属性的
@@ -103,10 +136,34 @@ class DownloadKit(object):
         self._missions_num += 1
         mission = Mission(self._missions_num, data)
         self._missions[self._missions_num] = mission
-        self._waiting_list.append(mission)
-        self._go()
+
+        thread_id = self._get_usable_thread()
+        if thread_id is not None:
+            thread = Thread(target=self._run, args=(thread_id, mission))
+            self._threads[thread_id] = {'thread': thread, 'mission': None}
+            thread.start()
+        else:
+            self._waiting_list.put(mission)
+
         return mission
 
+    def _run(self, ID: int, mission: 'Mission'):
+        while True:
+            if not mission:
+                if not self._waiting_list.empty():
+                    mission = self._waiting_list.get()
+                else:
+                    break
+
+            mission.info = '下载中'
+            mission.state = 'running'
+            self._threads[ID]['mission'] = mission
+            self._download(mission)
+            mission.state = 'done'
+            mission = None
+
+        self._threads[ID] = None
+
     def get_mission(self, mission_or_id: Union[int, 'Mission']) -> 'Mission':
         """根据id值获取一个任务                 \n
         :param mission_or_id: 任务或任务id
@@ -114,113 +171,86 @@ class DownloadKit(object):
         """
         return self._missions[mission_or_id] if isinstance(mission_or_id, int) else mission_or_id
 
-    def wait(self, mission: Union[int, 'Mission'], show: bool = True) -> tuple:
-        """等待一个任务完成           \n
-        :param mission: 任务对象或任务id
+    def wait(self, mission: Union[int, 'Mission'] = None, show: bool = True) -> Union[tuple, None]:
+        """等待所有或指定任务完成                                    \n
+        :param mission: 任务对象或任务id，为None时等待所有任务结束
         :param show: 是否显示进度
         :return: 任务结果和信息组成的tuple
         """
-        mission = self.get_mission(mission)
-        if show:
-            print(f'url：{mission.data["file_url"]}')
-            t1 = perf_counter()
-            while mission.file_name is None and perf_counter() - t1 < 4:
-                pass
-            print(f'文件名：{mission.file_name}')
-            print(f'目标路径：{mission.path}')
-
-        while mission.state != 'done':
+        if mission:
+            mission = self.get_mission(mission)
             if show:
-                info = mission.info
-                if isinstance(info, (float, int)):
-                    print(f'\r{info}% ', end='')
+                print(f'url：{mission.data["file_url"]}')
+                t1 = perf_counter()
+                while mission.file_name is None and perf_counter() - t1 < 4:
+                    sleep(0.01)
+                print(f'文件名：{mission.file_name}')
+                print(f'目标路径：{mission.path}')
 
-        if show:
-            if mission.result is False:
-                print(f'下载失败 {mission.info}')
-            elif mission.result is True:
-                print('\r100% ', end='')
-                print(f'下载完成 {mission.info}')
+            while mission.state != 'done':
+                if show:
+                    rate = mission.rate
+                    if isinstance(rate, (float, int)):
+                        print(f'\r{rate}% ', end='')
+                sleep(0.1)
+
+            if show:
+                if mission.result is False:
+                    print(f'下载失败 {mission.info}')
+                elif mission.result is True:
+                    print('\r100% ', end='')
+                    print(f'下载完成 {mission.info}')
+                else:
+                    print(f'已跳过 {mission.info}')
+                print()
+
+            return mission.result, mission.info
+
+        else:
+            if show:
+                self.show()
             else:
-                print(f'已跳过 {mission.info}')
-            print()
+                while self.is_running():
+                    sleep(0.1)
 
-        return mission.result, mission.info
-
-    def show(self) -> None:
-        if self._info_thread is None or not self._info_thread.is_alive():
-            self._info_thread = Thread(target=self._show)
-            self._info_thread.start()
+    def show(self, asyn: bool = False) -> None:
+        """实时显示所有线程进度             \n
+        :param asyn: 是否以异步方式显示
+        :return: None
+        """
+        if asyn:
+            Thread(target=self._show).start()
+        else:
+            self._show()
 
     def _show(self) -> None:
+        """实时显示所有线程进度"""
         t1 = perf_counter()
+        o = None
         while self.is_running() or perf_counter() - t1 < 2:
-            txt = [f'线程{k}:{v["mission"].info if v is not None else None}' for k, v in self._threads.items()]
-            print('\r' + '   '.join(txt), end='')
-        # from rich.progress import Progress
-        #
-        # with Progress() as progress:
-        #     for k, i in enumerate(self.threads.items()):
-        #         progress.add_task(f'[red]任务{k}', total=100)
-        #
-        #     while self.is_running() or perf_counter() - t1 < 2:
-        #         for k, i in self.threads.items():
-        #             if i is None:
-        #                 progress.update(k, completed=0)
-        #
-        #             else:
-        #                 new = self.threads[k]['mission'].info if self.threads[k] is not None else 0
-        #                 if isinstance(new, str):
-        #                     continue
-        #                 new = new or 0
-        #                 progress.update(k, completed=new)
-        #
-        #         sleep(0.02)
-
-    def _go(self) -> None:
-        """运行任务管理线程和线程管理线程"""
-        if self._mission_thread is None or not self._mission_thread.is_alive():
-            # print('任务线程启动', flush=False)
-            self._mission_thread = Thread(target=self._missions_manage)
-            self._mission_thread.start()
-
-        if self._manage_thread is None or not self._manage_thread.is_alive():
-            # print('管理线程启动', flush=False)
-            self._manage_thread = Thread(target=self._threads_manage)
-            self._manage_thread.start()
-
-    def _missions_manage(self) -> None:
-        """此方法是任务管理线程方法，用于把任务添加到空闲线程"""
-        t1 = perf_counter()
-        while self._waiting_list or perf_counter() - t1 < 2:
-            if self._waiting_list:
-                num = self._get_usable_thread()
-                msg = {'thread': None,
-                       'close': False,
-                       'mission': self._waiting_list.popleft()}
-                msg['mission'].state = 'running'
-                thread = Thread(target=self._download, args=(msg['mission'],))
-                msg['thread'] = thread
-                thread.start()
-                self._threads[num] = msg
-
-    def _threads_manage(self) -> None:
-        """此方法是线程管理方法，用于把完成的线程清除出列表"""
-        t1 = perf_counter()
-        while True:
+            if o:
+                print(f'\033[{self.size}A\r', end='')
+                print(f'\033[K', end='')
             for k, v in self._threads.items():
-                if isinstance(v, dict) and not v['thread'].is_alive():
-                    self._threads[k] = None
+                o = True
+                m = v['mission'] if v else None
+                rate, name, path = (f'{m.rate}%', m.file_name, m.path) if m else (None, None, None)
+                print(f'线程{k}：{rate} {path}{sep}{name}')
 
-            if perf_counter() - t1 > 2 and not self.is_running():
-                break
+            sleep(0.2)
 
-    def _get_usable_thread(self) -> int:
-        """获取可用线程"""
-        while True:
-            for k, v in self._threads.items():
-                if v is None:
-                    return k
+        print(f'\033[{self.size}A', end='')
+        for i in range(self.size):
+            print(f'\033[K', end='')
+            print(f'线程{i}：None')
+
+        print()
+
+    def _get_usable_thread(self) -> Union[int, None]:
+        """获取可用线程，没有则返回None"""
+        for k, v in self._threads.items():
+            if v is None:
+                return k
 
     def _download(self, mission: 'Mission') -> None:
         """此方法是执行下载的线程方法，用于根据任务下载文件     \n
@@ -245,14 +275,13 @@ class DownloadKit(object):
         goal_path = str(goal_Path)
         mission.path = goal_path
 
-        def set_result(res, info, state):
+        def set_result(res, info):
             mission.result = res
             mission.info = info
-            mission.state = state
 
         if file_exists == 'skip' and rename and (goal_Path / rename).exists():
             mission.file_name = rename
-            set_result(None, '已跳过', 'done')
+            set_result(None, '已跳过')
             return
 
         def do() -> tuple:
@@ -322,10 +351,9 @@ class DownloadKit(object):
                             if file_size:
                                 downloaded_size += 1024
                                 rate = downloaded_size / file_size if downloaded_size < file_size else 1
-                                mission.info = round(rate * 100, 2)
+                                mission.rate = round(rate * 100, 2)
 
             except Exception as e:
-                # raise
                 download_status, info = False, f'下载失败。\n{e}'
 
             else:
@@ -354,7 +382,7 @@ class DownloadKit(object):
                 if result[0] is not False:
                     break
 
-        set_result(result[0], result[1], 'done')
+        set_result(*result)
 
     def _make_response(self,
                        url: str,
@@ -451,7 +479,8 @@ class Mission(object):
         self._id = ID
         self.data = data
         self.state = 'waiting'
-        self.info = None
+        self.rate = 0
+        self.info = '等待下载'
         self.result = None
 
         self.file_name = None
@@ -508,9 +537,9 @@ def _get_download_file_name(url, response) -> str:
     return unquote(file_name, charset)
 
 
-def _get_session(session):
-    """
-    :param session: Union[Session, 'SessionOptions', 'MixPage', 'Drission']
+def _get_session(session: Union[Session, 'SessionOptions', 'MixPage', 'Drission']) -> Session:
+    """获取Session对象                                      \n
+    :param session: Session对象或包含Session信息的对象
     :return: Session对象
     """
     if not isinstance(session, Session):
